# Form implementation generated from reading ui file 'dwcopen.ui'
#
# Created by: PyQt6 UI code generator 6.7.0
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from pyqtgraph import QtCore, QtGui, QtWidgets


class Ui_DWCOpen(object):
    def setupUi(self, DWCOpen):
        DWCOpen.setObjectName("DWCOpen")
        DWCOpen.resize(564, 511)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(DWCOpen)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.txtPatientId = QtWidgets.QLineEdit(parent=DWCOpen)
        self.txtPatientId.setObjectName("txtPatientId")
        self.gridLayout.addWidget(self.txtPatientId, 0, 1, 1, 1)
        self.label = QtWidgets.QLabel(parent=DWCOpen)
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 0, 0, 1, 1)
        self.searchButton = QtWidgets.QPushButton(parent=DWCOpen)
        self.searchButton.setObjectName("searchButton")
        self.gridLayout.addWidget(self.searchButton, 0, 2, 1, 1)
        self.verticalLayout_2.addLayout(self.gridLayout)
        self.lblFound = QtWidgets.QLabel(parent=DWCOpen)
        self.lblFound.setText("")
        self.lblFound.setObjectName("lblFound")
        self.verticalLayout_2.addWidget(self.lblFound)
        self.cmbTypeofData = QtWidgets.QComboBox(parent=DWCOpen)
        self.cmbTypeofData.setObjectName("cmbTypeofData")
        self.cmbTypeofData.addItem("")
        self.cmbTypeofData.addItem("")
        self.verticalLayout_2.addWidget(self.cmbTypeofData)
        self.lstLabels = QtWidgets.QListWidget(parent=DWCOpen)
        self.lstLabels.setSelectionMode(QtWidgets.QAbstractItemView.SelectionMode.MultiSelection)
        self.lstLabels.setObjectName("lstLabels")
        self.verticalLayout_2.addWidget(self.lstLabels)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.label_2 = QtWidgets.QLabel(parent=DWCOpen)
        self.label_2.setObjectName("label_2")
        self.horizontalLayout_2.addWidget(self.label_2)
        self.dtFrom = QtWidgets.QDateTimeEdit(parent=DWCOpen)
        self.dtFrom.setCalendarPopup(True)
        self.dtFrom.setObjectName("dtFrom")
        self.horizontalLayout_2.addWidget(self.dtFrom)
        self.verticalLayout_2.addLayout(self.horizontalLayout_2)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.label_3 = QtWidgets.QLabel(parent=DWCOpen)
        self.label_3.setObjectName("label_3")
        self.horizontalLayout_3.addWidget(self.label_3)
        self.dtTo = QtWidgets.QDateTimeEdit(parent=DWCOpen)
        self.dtTo.setCalendarPopup(True)
        self.dtTo.setObjectName("dtTo")
        self.horizontalLayout_3.addWidget(self.dtTo)
        self.verticalLayout_2.addLayout(self.horizontalLayout_3)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.okButton = QtWidgets.QPushButton(parent=DWCOpen)
        self.okButton.setObjectName("okButton")
        self.horizontalLayout.addWidget(self.okButton)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.horizontalLayout.addItem(spacerItem1)
        self.cancelButton = QtWidgets.QPushButton(parent=DWCOpen)
        self.cancelButton.setObjectName("cancelButton")
        self.horizontalLayout.addWidget(self.cancelButton)
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.horizontalLayout.addItem(spacerItem2)
        self.verticalLayout_2.addLayout(self.horizontalLayout)

        self.retranslateUi(DWCOpen)
        self.cmbTypeofData.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(DWCOpen)

    def retranslateUi(self, DWCOpen):
        _translate = QtCore.QCoreApplication.translate
        DWCOpen.setWindowTitle(_translate("DWCOpen", "Load DWC data"))
        self.label.setText(_translate("DWCOpen", "Patient ID"))
        self.searchButton.setText(_translate("DWCOpen", "Search"))
        self.cmbTypeofData.setItemText(0, _translate("DWCOpen", "Numerics"))
        self.cmbTypeofData.setItemText(1, _translate("DWCOpen", "Waves"))
        self.label_2.setText(_translate("DWCOpen", "From"))
        self.label_3.setText(_translate("DWCOpen", "To"))
        self.okButton.setText(_translate("DWCOpen", "Ok"))
        self.cancelButton.setText(_translate("DWCOpen", "Cancel"))
